@@ -1,6 +1,7 @@
 /**************************************************************************
  FreeSockets - Portable C++ classes for IP(sockets) applications. (v0.3)
  Copyright (C) 2000-2001 Rafael Guterres Jeffman
+           (C) 2003-2006 Alistair Riddoch
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,7 +23,7 @@
  * This software package has been extensively modified by members of the
  * Worldforge Project. See the file ChangeLog for details.
  *
- * $Id: skserver.cpp,v 1.20 2005/09/15 10:38:56 alriddoch Exp $
+ * $Id: skserver.cpp,v 1.23 2006/10/12 01:10:12 alriddoch Exp $
  *
  */
 
@@ -38,6 +39,7 @@
 #include <errno.h>
 #endif // _WIN32
 
+#include <iostream>
 #include <cstdio>
 
 #ifdef _WIN32
@@ -62,34 +64,20 @@ static inline int closesocket(SOCKET_TYPE sock)
 }
 #endif // HAVE_CLOSESOCKET
 
+#ifdef HAVE_GETADDRINFO
+#ifndef HAVE_GAI_STRERROR
+const char * skstream_gai_strerror(int errcode);
+
+static const char * gai_strerror(int errcode)
+{
+    return skstream_gai_strerror(errcode);
+}
+#endif // HAVE_GAI_STRERROR
+#endif // HAVE_GETADDRINFO
+
 /////////////////////////////////////////////////////////////////////////////
 // class basic_socket_server implementation
 /////////////////////////////////////////////////////////////////////////////
-// sistem dependant initialization
-bool basic_socket_server::startup() {
-  #ifdef _WIN32
-    const unsigned wMinVer = 0x0101;	// request WinSock v1.1 (at least)
-    WSADATA wsaData;
-    LastError = WSAStartup(wMinVer, &wsaData);
-    return (LastError == 0);
-  #else
-    return true;
-  #endif
-}
-
-// sistem dependant finalization
-void basic_socket_server::shutdown() {
-  if(is_open()) {
-    if(::shutdown(_socket, SHUT_RDWR) == SOCKET_ERROR) {
-      setLastError();
-    }
-  }
-}
-
-// sistem dependant error verification
-void basic_socket_server::setLastError() {
-    LastError = getSystemError();
-}
 
 basic_socket_server::~basic_socket_server() {
   basic_socket_server::close();
@@ -118,6 +106,14 @@ void basic_socket_server::close() {
   }
 }
 
+void basic_socket_server::shutdown() {
+  if(is_open()) {
+    if(::shutdown(_socket, SHUT_RDWR) == SOCKET_ERROR) {
+      setLastError();
+    }
+  }
+}
+
 bool basic_socket_server::can_accept() {
   if(_socket == INVALID_SOCKET) return false;
 
@@ -146,7 +142,7 @@ bool basic_socket_server::can_accept() {
 // class ip_socket_server implementation
 /////////////////////////////////////////////////////////////////////////////
 
-bool ip_socket_server::bindToIpService(int service, int type, int protocol)
+int ip_socket_server::bindToIpService(int service, int type, int protocol)
 {
 #ifdef HAVE_GETADDRINFO
   struct addrinfo req, *ans;
@@ -168,12 +164,12 @@ bool ip_socket_server::bindToIpService(int service, int type, int protocol)
     std::cout << "skstream: " << gai_strerror(ret)
               << std::endl << std::flush;
     setLastError();
-    return false;
+    return -1;
   }
 
-  bool success = false;
+  int success = -1;
 
-  for(struct addrinfo * i = ans; success == false && i != 0; i = i->ai_next) {
+  for(struct addrinfo * i = ans; success == -1 && i != 0; i = i->ai_next) {
     _socket = ::socket(i->ai_family, i->ai_socktype, i->ai_protocol);
     if (_socket == INVALID_SOCKET) {
       setLastError();
@@ -188,13 +184,13 @@ bool ip_socket_server::bindToIpService(int service, int type, int protocol)
       setLastError();
       close();
     } else {
-      success = true;
+      success = 0;
     }
   }
 
   ::freeaddrinfo(ans);
 
-  return success;
+  return 0;
 #else
   // create socket
   _socket = ::socket(AF_INET, type, protocol);
@@ -250,7 +246,7 @@ bool tcp_socket_server::open(int service)
     close();
   }
 
-  if (!bindToIpService(service, SOCK_STREAM, IPPROTO_TCP)) {
+  if (bindToIpService(service, SOCK_STREAM, IPPROTO_TCP) != 0) {
     return false;
   }
 
@@ -272,11 +268,6 @@ udp_socket_server::~udp_socket_server()
 {
 }
 
-SOCKET_TYPE udp_socket_server::accept()
-{
-    return _socket;
-}
-
 // create a UDP socket binded to a given port
 bool udp_socket_server::open(int service)
 {
@@ -284,7 +275,7 @@ bool udp_socket_server::open(int service)
     close();
   }
 
-  if (!bindToIpService(service, SOCK_DGRAM, IPPROTO_UDP)) {
+  if (bindToIpService(service, SOCK_DGRAM, IPPROTO_UDP) != 0) {
     return false;
   }
 
